//
// Written by Igor Tolmachev, IT Samples
//        mailto:support@itsamples.com
//
// Copyright (c) 2009.
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name and all copyright 
// notices remains intact. If the source code in this file is used in 
// any  commercial application then a statement along the lines of 
// "Portions copyright (c) Igor Tolmachev, 2009" must be included in 
// the startup banner, "About" box or printed documentation. An email 
// letting me know that you are using it would be nice as well. That's 
// not much to ask considering the amount of work that went into this.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability for any damage/loss of business that
// this product may cause.
//
// Expect bugs!
// 
// Please use and enjoy, and let me know of any bugs/mods/improvements 
// that you have found/implemented and I will fix/incorporate them into 
// this file. 

// RDPStorage.cpp: implementation of the CRDPStorage class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RDPRecovery.h"
#include "RDPStorage.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define PASWD_LEN 8192
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRDPStorage::CRDPStorage()
{
	m_hWndParent = NULL;
}

CRDPStorage::~CRDPStorage()
{

}

void CRDPStorage::SetMessageTarget(HWND hWnd)
{
	m_hWndParent = hWnd;
}

void CRDPStorage::PostLastMessage(char* pszMessage)
{
	if(m_hWndParent)
		SendMessage(m_hWndParent, WM_POSTLASTMESSAGE, 0, reinterpret_cast<LPARAM>(pszMessage));
}

BOOL CRDPStorage::LoadFromFile(char *pszPath)
{
	FILE* pFile = fopen(pszPath,"rb");
	if(pFile)
	{
		WCHAR wszLine[PASWD_LEN];
		TCHAR szLine[PASWD_LEN];
		
		memset(szLine,0,PASWD_LEN);
		memset(wszLine,0,PASWD_LEN*2);

		CStringArray arValues;
		WCHAR wcUnicode = fgetwc(pFile);

		TCHAR szUser[512];
		memset(szUser,0,512);
		
		TCHAR szAddress[512];
		memset(szAddress,0,512);

		if(wcUnicode == 0xFEFF)
		{
			while(fgetws(wszLine, PASWD_LEN, pFile) != NULL)
			{
				if(wcsstr(wszLine, L"pass"))
				{
					WideCharToMultiByte(CP_ACP, 0, wszLine, -1, szLine, sizeof(szLine), NULL, NULL);
					arValues.Add(CString(szLine));
				}
				else if(wcsstr(wszLine, L"username"))
					WideCharToMultiByte(CP_ACP, 0, wszLine, -1, szUser, sizeof(szLine), NULL, NULL);
				else if(wcsstr(wszLine, L"address"))
					WideCharToMultiByte(CP_ACP, 0, wszLine, -1, szAddress, sizeof(szLine), NULL, NULL);

				memset(szLine,0,PASWD_LEN);
				memset(wszLine,0,PASWD_LEN*2);
			}
		}
		else
		{
			fseek(pFile,0L,SEEK_SET);
			while(fgets(szLine, PASWD_LEN, pFile) != NULL)
			{
				if(strstr(szLine, "pass"))
					arValues.Add(CString(szLine));
				else if(strstr(szLine, "username"))
					strcpy(szUser, szLine);
				else if(strstr(szLine, "address"))
					strcpy(szAddress, szLine);

				memset(szLine,0,PASWD_LEN);
			}
		}

		if(arValues.GetSize() > 0)
		{
			memset(szLine, 0, PASWD_LEN);
			strcpy(szLine, pszPath);
			strcat(szLine, "\r\n---------------------------------------------------------");
			PostLastMessage(szLine);

			if(strlen(szAddress) > 0 || strlen(szUser) > 0)
			{
				memset(szLine, 0, PASWD_LEN);
				if(strlen(szAddress) > 0)
				{
					if(strstr(szAddress, "full address:s:"))
						strcat(szLine, strstr(szAddress, "full address:s:") + 15);
					else
						strcat(szLine, szAddress);
					//strcat(szLine, "\r\n");
				}

				if(strlen(szUser) > 0)
				{
					if(strstr(szUser, "username:s:"))
						strcat(szLine, strstr(szUser, "username:s:") + 11);
					else
						strcat(szLine, szUser);
					//strcat(szLine, "\r\n");
				}

				strcat(szLine, "---------------------------------------------------------");
				PostLastMessage(szLine);
			}


			for(int i=0; i < arValues.GetSize(); i++)
			{
				memset(szLine, 0, PASWD_LEN);
				strcpy(szLine, arValues.GetAt(i));
				Decrypt(szLine);
				PostLastMessage("---------------------------------------------------------");
			}
		}
		else
		{
			PostLastMessage("Selected file does not contain RDP passwords");

			fclose(pFile);
			return FALSE;
		}

		fclose(pFile);
	}

	return TRUE;
}

int CRDPStorage::Decrypt(char* pszData) 
{
	DATA_BLOB blobDataIn;
	DATA_BLOB blobDataOut;
	ZeroMemory(&blobDataIn, sizeof(blobDataIn));
	ZeroMemory(&blobDataOut, sizeof(blobDataOut));

	TCHAR szLine[PASWD_LEN];
	memset(szLine,0,PASWD_LEN);

	if(strstr(pszData, "="))
		strcpy(szLine, strstr(pszData, "=") + 1);
	else if(strstr(pszData, "51:b:"))
		strcpy(szLine, strstr(pszData, "51:b:") + 5);
	else
		strcpy(szLine, pszData);

	Hex2Blob(szLine, &blobDataIn); 

	LPWSTR lpwsDescription = NULL;

	if(!CryptUnprotectData(
	  &blobDataIn,						// input data
	  &lpwsDescription, 				// output description
	  NULL, 							// optional entropy
	  NULL, 							// reserved
	  NULL, 							// optional prompt structure
	  CRYPTPROTECT_UI_FORBIDDEN,		// flags
	  &blobDataOut))
	{
		PostLastMessage("The input is invalid or you can't decrypt it because the data has been encrypted on another system or with another account.");
		return FALSE;
	}

	if(blobDataOut.pbData)
	{
		memset(szLine,0,PASWD_LEN);
		WideCharToMultiByte(CP_ACP, 0, (USHORT*)blobDataOut.pbData, -1, szLine, sizeof(szLine), NULL, NULL);

		TCHAR szResult[PASWD_LEN];
		memset(szResult,0,PASWD_LEN);
		sprintf(szResult, "password:\t%s", szLine);
		PostLastMessage(szResult);

		if(lpwsDescription) 
		{
			memset(szLine,0,PASWD_LEN);
			WideCharToMultiByte(CP_ACP, 0, (USHORT*)lpwsDescription, -1, szLine, sizeof(szLine), NULL, NULL);

			sprintf(szResult, "description:\t%s", szLine);
			PostLastMessage(szResult);

			LocalFree(lpwsDescription);
		}

		if(blobDataOut.pbData)  LocalFree(blobDataOut.pbData);
		if(blobDataIn.pbData)   LocalFree(blobDataIn.pbData);
	}

	return TRUE;
}

void CRDPStorage::Hex2Blob(char* pszIn, DATA_BLOB* blobOut)
{
	int nLen = strlen(pszIn) / 2;
	blobOut->pbData = (UCHAR*)LocalAlloc(LPTR, nLen);

	if(!blobOut->pbData)
	{
		blobOut->cbData = 0;
		return;
	}

	char* p = (char*)blobOut->pbData;

	while(pszIn[0] && pszIn[1] && (sscanf(pszIn, "%02x", &nLen) == 1))
	{
		*p++ = nLen;
		pszIn += 2;
	}

	blobOut->cbData = p - (char*)blobOut->pbData;
}

